package top.xinsin.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.AccountMapper;
import top.xinsin.pojo.Account;
import com.baomidou.mybatisplus.extension.service.IService;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务类
 * </p>
 *
 * @author xinsin
 * @since 2023-04-12
 */
@Service
public class AccountService implements UserDetailsService {

    @Autowired
    private AccountMapper accountMapper;

    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        System.out.println(username);
        if(username == null) throw new UsernameNotFoundException("用户名不能为空");
        Account account = accountMapper.selectOne(new LambdaQueryWrapper<Account>()
                .eq(Account::getUsername,username)
                .or()
                .eq(Account::getEmail,username)
                .eq(Account::getStatus,true));
        if(account == null) throw new UsernameNotFoundException("用户名或密码错误");
        return User
                .withUsername(account.getUsername())
                .password(account.getPassword())
                .roles("role=" + account.getRoleId(),"id=" + account.getId())
                .build();
    }

    public R<String> register(Account account) {
        account.setCreateTime(new Date());
        account.setStatus(true);
        account.setPassword(encoder.encode(account.getPassword()));
        int insert = accountMapper.insert(account);
        return insert == 1 ? R.success("注册成功") : R.failed(HttpCodes.HTTP_CODES555,"注册失败,您的用户名或者邮箱已经被注册过了");
    }
    public R<String> changeAccount(Account account) {
        if (account.getPassword() != null){
            account.setPassword(encoder.encode(account.getPassword()));
        }
        int update = accountMapper.updateById(account);
        return update == 1 ? R.success("修改用户信息成功") : R.failed(HttpCodes.HTTP_CODES555,"修改用户信息失败,请联系服务器管理员");
    }

    public R<String> changeStatus(Integer userId, Boolean status) {
        Account account = new Account();
        account.setId(userId);
        account.setStatus(status);
        int i = accountMapper.updateById(account);
        if (i == 1){
            if (status){
                return R.success("启用该用户成功");
            }else{
                return R.success("禁用该用户成功");
            }
        }else{
            return R.failed(HttpCodes.HTTP_CODES555,"修改用户状态失败");
        }
    }

    public R<String> changePassword(Integer userId, String oldPassword, String newPassword) {
        Account account = new Account();
        account.setPassword(encoder.encode(newPassword));
        LambdaQueryWrapper<Account> accountLambdaQueryWrapper = new LambdaQueryWrapper<>();
        accountLambdaQueryWrapper
                .eq(Account::getId,userId)
                .eq(Account::getPassword,encoder.encode(oldPassword));
        int update = accountMapper.update(account, accountLambdaQueryWrapper);
        return update == 1 ? R.success("修改密码成功") : R.failed(HttpCodes.HTTP_CODES555,"修改密码失败,请联系服务器管理员");
    }

    public R<JSONObject> getUser(Integer page, Integer num) {
        Page<Account> accountPage = new Page<>(page,num);
        Page<Account> accountPage1 = accountMapper.selectPage(accountPage, null);
        JSONObject jsonObject = new JSONObject();
        List<Account> collect = accountPage1.getRecords().stream().peek(e -> e.setPassword(null)).toList();
        jsonObject
                .fluentPut("info",collect)
                .fluentPut("total",accountPage1.getTotal());
        return R.success(jsonObject);
    }

    public R<String> delUser(Integer id) {
        int delete = accountMapper.deleteById(id);
        return delete == 1 ? R.success("删除用户成功") : R.failed(HttpCodes.HTTP_CODES555,"删除用户失败,请联系服务器管理员");
    }
}
